/*
 * @Author: wangcb
 * @Email: wangcbgg@163.com
 * @Date: 2021-11-01 10:38:11
 * @LastEditTime: 2022-04-06 15:36:48
 */

const figlet = require('figlet');
const download = require('download-git-repo');
const { prompt } = require('inquirer');
const { pathExists } = require('fs-extra');
const { redBright, blueBright, greenBright, cyan } = require('chalk');
const ora = require('ora');
const { spawn } = require('child_process');
const path = require('path');

/**
 * 创建问题清单
 * @returns {Promise<{projectName:string,template:'Sepg (ES6)'|'Snel (Vue)'|'react'}>}
 */
async function setQuestion() {
  const answer = await prompt([
    {
      name: 'projectName',
      message: 'Input a project name',
      default: 'epg-project',
      validate: async val => {
        const exist = await pathExists(val);
        if (exist) return redBright.bold('the project name already exists');
        return true;
      }
    }, {
      name: 'template',
      type: 'list',
      message: 'Select a template',
      choices: ['Sepg (ES6)', 'Snel (Vue)']
    }
  ]);
  return answer;
}
/**
 * 创建剩余问题清单
 * @returns {Promise<{webpack:'webpack5'|'webpack4',platform:'hw_v6'|'zte'|'none',pkgt:'yarn'|'npm'}>}
 */
async function setRestQuestion(webpack) {
  const question = [
    {
      name: 'pkgt',
      type: 'list',
      message: 'Pick the package manager to use when installing dependencies',
      choices: ['yarn', 'npm']
    }
  ];
  webpack && question.unshift({
    name: 'webpack',
    type: 'list',
    message: 'Select the webpack version',
    choices: ['webpack5', 'webpack4'],
    default: 'webpack5'
  }, {
    name: 'platform',
    type: 'list',
    message: 'Select a basic api platform',
    choices: ['hw_v6', 'zte', 'none']
  });
  const answer = await prompt(question);
  return answer;
}
/** 创建进度显示器 */
function createSpinner() {
  const spinner = ora({
    text: blueBright('Downloading template...'),
    spinner: {
      interval: 180,
      frames: ['---', '^--', '-^-', '--^', '---']
    },
  });
  return spinner;
}
/**
 * 开始拉取仓库模板
 * @param {{projectName:string,template:'Sepg (ES6)'|'Snel (Vue)'|'react',webpack:'webpack5'|'webpack4',platform:'hw_v6'|'zte'|'none',pkgt:'yarn'|'npm'}} answer 用户选择
 * @param {ora.Ora} spinner 进度显示器
 */
function startDownload(answer, spinner) {
  const { projectName, template, webpack, platform, pkgt } = answer;
  let registry = `git@gitee.com:wangcbgg/sepg_${webpack}.git`;
  if (template === 'Snel (Vue)') {
    registry = 'git@gitee.com:wangcbgg/snel.git';
  } else {
    if (platform !== 'none') {
      registry = `${registry}#${platform}`;
    }
  }
  try {
    download(`direct:${registry}`, `./${projectName}`, { clone: true }, async err => {
      if (err) {
        spinner.fail(redBright('Exception'));
        console.log('\n', err);
      } else {
        spinner.succeed();
        await installDependencies(answer, spinner);
        console.log(`\r\nSuccessfully created project ${cyan(projectName)}`);
        console.log(`\r\n  cd ${cyan(projectName)}`);
        console.log(pkgt === 'npm' ? '  npm run dev\r\n' : '  yarn dev\r\n');
      }
    });
  } catch (error) {
    spinner.fail(redBright('Exception'));
    console.log('\n', error);
  }
}
/** 
 * 安装依赖
 * @param {{projectName:string,template:'Sepg (ES6)'|'Snel (Vue)'|'react',webpack:'webpack5'|'webpack4',pkgt:'yarn'|'npm'}} answer 用户选择
 * @param {ora.Ora} spinner 进度显示器
 */
async function installDependencies(answer, spinner) {
  const { projectName, pkgt } = answer;
  spinner.start(greenBright('Installing dependencies...'));
  try {
    return new Promise(resolve => {
      const target = path.join(process.cwd(), projectName);
      const command = pkgt + (process.platform === 'win32' ? '.cmd' : '');
      const args = pkgt === 'npm' ? ['install', '--legacy-peer-deps'] : null;
      const p = spawn(command, args, { cwd: target });
      p.on('exit', () => {
        spinner.succeed();
        resolve();
      });
      p.on('error', err => {
        console.log('err: ', err);
        spinner.fail();
      });
    });
  } catch (err_1) {
    console.log('err: ', err_1);
    spinner.fail();
  }
}
/** 初始化 */
function init() {
  figlet.text('#` EPG - CIL', {
    font: 'Slant'
  }, async (err, data) => {
    if (err) {
      console.log(redBright.bold(err));
      return;
    }
    console.log(`${greenBright.bold(data)}\n\t\t ${cyan('Ctrl+C')} to exit the program\n`);
    const answer = await setQuestion();
    const { template } = answer;
    const restAnswer = await setRestQuestion(template === 'Sepg (ES6)');
    const spinner = createSpinner().start();
    startDownload(Object.assign(answer, restAnswer), spinner);
  });
}

module.exports = init;